/*
 *	DraggedIconView.m
 *
 *	Ver.2.0  1995-04-08	based on NXDraggingInfo protocol
 *		by  Takeshi Ogihara (ogihara@seg.kobe-u.ac.jp)
 *	Ver.3.0  1999-03-21	for OPENSTEP
 *		(based on NSDraggingInfo, NSDraggingDestination)
 *		by  Takeshi Ogihara (ogihara@seg.kobe-u.ac.jp)
 */

#import "DraggedIconView.h"
#import <AppKit/NSImage.h>
#import <AppKit/NSEvent.h>
#import <AppKit/NSPasteboard.h>
#import <Foundation/NSArray.h>
#import "../FileSelector.h"

#define  folderIMAGE		@"folder"
#define  openFolderIMAGE	@"openFolder"

@implementation DraggedIconView

static NSArray *pbType = nil;
static NSArray *wplist = nil;
static NSImage *folderImage, *openFolderImage;

+ (void)initialize
{
	pbType = [[NSArray alloc]
		initWithObjects: NSFilenamesPboardType, nil];
	folderImage = [NSImage imageNamed:folderIMAGE];
	openFolderImage = [NSImage imageNamed:openFolderIMAGE];
}

- (id)setup
{
	[self registerForDraggedTypes:pbType];
	[self clear];
	return self;
}

- (void)clear
{
	iconimage = folderImage;
	backgray = NSLightGray;
	[self display];
}

- (void)setHighlight:(BOOL) flag
{
	// backgray = flag ? NSWhite : NSLightGray;
	iconimage = flag ? openFolderImage : folderImage;
	[self display];
}

- (unsigned int)fileOperation
{
	return file_op;
}

/* Local Method */
- (unsigned int)checkDragOperation:(id <NSDraggingInfo>)sender
{
	unsigned int sourceDragMask = [sender draggingSourceOperationMask];
	/* Value of sourceDragMask ... */
	/* without any key:	07 */
	/* with alternate key:	01 = NSDragOperationCopy */
	/* with control key:	02 = NSDragOperationLink */
	/* with command key:	04 = NSDragOperationGeneric */
	if (sourceDragMask == NSDragOperationGeneric)
		return (file_op = NSDragOperationGeneric);
	if (sourceDragMask == NSDragOperationLink)
		return (file_op = NSDragOperationLink);
	if (sourceDragMask & NSDragOperationCopy)
		return (file_op = NSDragOperationCopy);
	return (file_op = NSDragOperationNone);
}

- (unsigned int) draggingEntered:(id <NSDraggingInfo>)sender
{
	[self setHighlight: YES];
	// [controller iconEntered: self];
	return [self checkDragOperation: sender];
}

- (unsigned int) draggingUpdated:(id <NSDraggingInfo>)sender
{
	return [self checkDragOperation: sender];
}

- (void)draggingExited:(id <NSDraggingInfo>)sender
{
	// [controller iconExited: self];
	[self setHighlight: NO];
}

- (BOOL) prepareForDragOperation:(id <NSDraggingInfo>)sender
{
	id	plist;
	NSPasteboard *pboard = [sender draggingPasteboard];

	if (![[pboard availableTypeFromArray: pbType]
			isEqualToString: NSFilenamesPboardType])
		return NO;
	if ((plist = [pboard propertyListForType:NSFilenamesPboardType]) == nil)
		return NO;
	if ([plist isKindOfClass:[NSString class]])
		plist = [NSArray arrayWithObject: plist];
	else if (![plist isKindOfClass:[NSArray class]])
		return NO;

	if (wplist) [wplist release];
	wplist = [[NSArray alloc] initWithArray: plist];
	[self clear];
	return YES;
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
	return YES;
}

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
{
	[controller iconReleased:self data:wplist];
	// This message must be called here,
	// because it uses Workspace Operations.
	[wplist release];
	wplist = nil;
	[self display];
}

- (void)drawRect:(NSRect)r
{
	PSsetgray(backgray);
	NSRectFill(r);
	[iconimage compositeToPoint:(r.origin) fromRect:r
		operation: NSCompositeSourceOver];
}

@end
