#import "../FileSelector.h"
#import <Foundation/NSArray.h>
#import <Foundation/NSFileManager.h>
#import <AppKit/NSBrowser.h>
#import <AppKit/NSEvent.h>
#import <AppKit/NSWorkspace.h>
#import <libc.h>
#import "DraggedIconView.h"
#import "../TextImageCreator.h"
#import "../ScenarioCtrl.h"
#import "../PrefCtrl.h"

#define  SelectorPanelName	@"FileSelector"
#define  LocBias	18
static NSPoint prevOrigin = { 0.0, 0.0 };

@implementation FileSelector

- (void)dealloc
{
	[panel saveFrameUsingName: SelectorPanelName];
	[panel close];
	[panel release];
	// Note: panel is not released automatically.
	// Note: iconview is released when the panel is closed.
	[rootPath release];
	[cwd release];
	[super dealloc];
}

- (void)activateAndOrderFront:(id)sender
{
	id title;

	[cwd release];
	cwd = @"";
	cwdCol = 0;
	if (panel == nil) {
		[NSBundle loadNibNamed:@"FileSelector.nib" owner:self];
		// [panel setDelegate:self];
		[panel setFrameUsingName: SelectorPanelName];
		[iconView setup];
		[iconText setStringValue:[rootPath lastPathComponent]];
		[browser setTitle:[rootPath lastPathComponent] ofColumn:0];
	}
	[browser setPath:@"/"];
	[browser setTarget:self];
	[browser setAction:@selector(fileSelected:)];
	[browser setDoubleAction:@selector(fileDoubleClicked:)];
	title = [NSString stringWithFormat:@"%@ : %@",
		NSLocalizedString(@"Select Files", SelectFiles),
		[[scenario filename] lastPathComponent]];
	[panel setTitle:title];
	if (prevOrigin.x == 0.0 && prevOrigin.y == 0.0)
		prevOrigin = [panel frame].origin;
	else {
		static int serNo = 0;
		NSPoint pt;
		pt.x = prevOrigin.x + LocBias * (++serNo % 3);
		pt.y = prevOrigin.y + LocBias * (serNo % 2);
		[panel setFrameOrigin:pt];
	}
	[panel makeKeyAndOrderFront:self];
	// [panel setFloatingPanel:YES];	...if you need a floating panel
	// [NSApp runModalForWindow:panel];	...if you need a modal panel
}

- (void)setScenarioCtrl:(id)newScenario
{
	scenario = newScenario;
}

- (BOOL)setPath:(NSString *)path
{
	[rootPath release];
	[(rootPath = path) retain];
	return YES;
}

/* Local Method */
- (NSMutableArray *)getPathAndComponent
{
	NSMutableArray *comp;
	NSString *iconp;
	int snum, i;
	int cnum = [browser selectedColumn];
	id  cell = [browser selectedCells];
	if (cell == nil || (snum = [cell count]) == 0)
		return nil;
	[cwd release];
	cwdCol = cnum;
	if (cnum == 0)
		cwd = @"";
	else
		cwd = [browser pathToColumn:cnum];
	if ([[cell objectAtIndex:0] isLeaf]) {
		comp = [NSMutableArray arrayWithCapacity:snum];
		for (i = 0; i < snum; i++)
			[comp addObject:[[cell objectAtIndex:i] stringValue]];
	}else { /* directory */
		NSString *w = [[cell objectAtIndex:0] stringValue];
		if ([cwd length] == 0)
			cwd = w;
		else
			cwd = [cwd stringByAppendingPathComponent:w];
		comp = nil;
		++cwdCol;
	}
	[cwd retain];
	iconp = ([cwd length] == 0) ? rootPath : cwd;
	[iconText setStringValue:[iconp lastPathComponent]];
	return comp;
}

- (void)addSelectedFile:(id)sender
{
	id	arr, added;

        if ((arr = [self getPathAndComponent]) == nil) {
		if (cwdCol == 0) {	/* root */
			NSBeep();	/* Nothing selected */
			return;
		}
		/* directory is selected */
		added = [NSArray arrayWithObject: cwd];
	}else if ([cwd length] > 0) { /* in sub-directory */
		id	pa;
		NSString *ps, *obj;
		int	i, n = [arr count];
		pa = [[cwd pathComponents] mutableCopy];
		obj = [pa objectAtIndex:0];
		if ([obj length] == 0 || [obj isEqualToString:@"/"]) {
			[pa removeObjectAtIndex:0];
			ps = [NSString pathWithComponents:pa];
		}else
			ps = cwd;
		added = [NSMutableArray arrayWithCapacity:n];
		for (i = 0; i < n; i++) {
			obj = [ps stringByAppendingPathComponent:[arr objectAtIndex:i]];
			[added addObject:obj];
		}
	}else /* in root directory */
		added = arr;
	// [NSApp stopModal];	...if you need a modal panel
	[scenario makeKeyAndOrderFront:self];
	[scenario addFiles:added];
}

/* Local Method */
- (void)displayBrowser:(id)brw
{
	[brw reloadColumn:cwdCol];
	[brw displayColumn:cwdCol];
}

- (void)deletePushed:(id)sender
{
	NSString  *src, *fpath;
	id	arr;
	int	tag;

	fpath = [rootPath stringByAppendingPathComponent:cwd];
        if ((arr = [self getPathAndComponent]) == nil) {
		if (cwdCol == 0) {	/* root */
			NSBeep();	/* Nothing selected */
			return;
		}
		src = [fpath stringByDeletingLastPathComponent];
		arr = [NSArray arrayWithObject:[fpath lastPathComponent]];
		--cwdCol;
	}else
		src = fpath;
	[[NSWorkspace sharedWorkspace]
			performFileOperation:NSWorkspaceRecycleOperation
			source:src destination:@"" files:arr tag:&tag];
	sync();
	[self performSelector:@selector(displayBrowser:)
			withObject:browser afterDelay: 2];
	/* Reload of browser would fail immediately after file operation,
	   because we can not know the change of file system so soon. */
}

- (void)redisplay:(id)sender
{
	[self displayBrowser:browser];
}

- (void)fileSelected:(id)sender
{
	(void)[self getPathAndComponent];
}

- (void)fileDoubleClicked:(id)sender
{
	NSString *fpath, *fn;
	id	arr = [self getPathAndComponent];
	fpath = [rootPath stringByAppendingPathComponent:cwd];
	fn = (arr == nil) ? fpath
		: [fpath stringByAppendingPathComponent:[arr objectAtIndex:0]];
	[[NSWorkspace sharedWorkspace] openFile: fn];
}

- (void)browser:(NSBrowser *)sender createRowsForColumn:(int)column inMatrix:(NSMatrix *)matrix
{
	int	idx, num;
	NSString *name, *path, *dirp;
	id	manager, cel;
	NSArray	*dcont, *sortarr;
	NSMutableArray *dotarr, *normarr;
	BOOL	isdir, dotted;

	dotted = [[PrefCtrl defaultPref] isDottedFileListed];
	if (column == 0)
		dirp = rootPath;
	else
		dirp = [rootPath stringByAppendingPathComponent:
			[browser pathToColumn:column]];
	manager = [NSFileManager defaultManager];
	dcont = [manager directoryContentsAtPath:dirp];
	num = [dcont count];
	normarr = [NSMutableArray arrayWithCapacity:1];
	dotarr = [NSMutableArray arrayWithCapacity:1];
	for (idx = 0; idx < num; idx++) {
		name = [dcont objectAtIndex: idx];
		if ([name characterAtIndex:0] != '.')
			[normarr addObject:name];
		else if (dotted)
			[dotarr addObject:name];
	}
	sortarr = [normarr sortedArrayUsingSelector:
			@selector(caseInsensitiveCompare:)];
	if (dotted) {
		id t = [dotarr sortedArrayUsingSelector:
				@selector(caseInsensitiveCompare:)];
		sortarr = [sortarr arrayByAddingObjectsFromArray:t];
	}

	num = [sortarr count];
	for (idx = 0; idx < num; idx++) {
		name = [sortarr objectAtIndex: idx];
		[matrix renewRows:idx+1 columns:1];
		cel = [matrix cellAtRow:idx column:0];
		[cel setStringValue:name];
		[cel setLoaded:YES];
		path = [dirp stringByAppendingPathComponent:name];
		if ([TextImageCreator isBinaryFile:name]) {
			[cel setLeaf:YES];
			[cel setEnabled:NO];
			continue;
		}
		if (![manager fileExistsAtPath:path isDirectory:&isdir]) {
			/* File does not exist ?? */
			[cel setLeaf:YES];
			[cel setEnabled:NO];
		}else if (isdir)
			[cel setLeaf:[TextImageCreator isWrappedFile:name]];
		else /* plain file */
			[cel setLeaf: YES];
	}
	[matrix setNeedsDisplay:YES];
}


/***** ICON VIEW METHODS ****************************/
- (BOOL)iconReleased:(id)sender data:(NSArray *)plist
{
	int	n, i, tag;
	id 	ws;
	unsigned int	op = [sender fileOperation];
	NSString *opstr = nil;
	NSString *dragpath;
	NSMutableArray *flist;
	NSString *fpath = [rootPath stringByAppendingPathComponent:cwd];

	switch (op) {
	case NSDragOperationGeneric:
		opstr = NSWorkspaceMoveOperation;
		break;
	case NSDragOperationLink:
		opstr = NSWorkspaceLinkOperation;
		break;
	default:
		opstr = NSWorkspaceCopyOperation;
		break;
	}
	flist = [NSMutableArray arrayWithCapacity:[plist count]];
	n = [plist count];
	for (i = 0; i < n; i++)
		[flist addObject:[[plist objectAtIndex:i] lastPathComponent]];
	dragpath = [[plist objectAtIndex:0] stringByDeletingLastPathComponent];
	ws = [NSWorkspace sharedWorkspace];
	if (![ws performFileOperation:opstr source:dragpath
			destination:fpath files:flist tag:&tag]) {
		NSBeep();
		return NO;
	}
	sync();
	[self performSelector:@selector(displayBrowser:)
		withObject:browser afterDelay: 2];
	/* Reload of browser would fail immediately after file operation,
	   because we can not know the change of file system so soon. */

	return YES;
}

/* NSWinodow's Delegate */
- (BOOL)windowShouldClose:(id)sender
{
	[scenario makeKeyAndOrderFront:self];
	return YES;
}

@end
