#import "FullScreenView.h"
#import <AppKit/NSApplication.h>
#import <AppKit/NSWindow.h>
#import <AppKit/NSImage.h>
#import <AppKit/dpsOpenStep.h>
#import <AppKit/NSColor.h>
#import <AppKit/NSEvent.h>
#import <AppKit/psopsOpenStep.h>
#import "NSColorAdditional.h"
#import "PlayControl.h"
#import "PrefCtrl.h"
#import "sheetInfo.h"
#import "PSsplat.h"

#define  PSsplat(r, p)   _PSsplat( (r)->origin.x, (r)->origin.y, \
		(r)->size.width, (r)->size.height, (p)->x, (p)->y )
#define  TooLargeRatio	2.2

#define CursorImgR @"CRightArrow.tiff"
#define CursorImgL @"CLeftArrow.tiff"
#define CursorImgP @"CPause.tiff"

static NSCursor *CursorR = nil;
static NSCursor *CursorL = nil;
static NSCursor *CursorP = nil;

@implementation FullScreenView

+ (void)initialize
{
	NSPoint spot;

	spot.x = 7.0;
	spot.y = 7.0;
	CursorR = [NSCursor alloc];
	CursorL = [NSCursor alloc];
	CursorP = [NSCursor alloc];
#if 0	/* Not Implemented yet...? */
        [xCursor initWithImage: [NSImage imageNamed:CrossCursor]];
	[xCursor setHotSpot:spot];
#else
	[CursorL initWithImage: [NSImage imageNamed:CursorImgL] hotSpot:spot];
	[CursorP initWithImage: [NSImage imageNamed:CursorImgP] hotSpot:spot];
	[CursorR initWithImage: [NSImage imageNamed:CursorImgR] hotSpot:spot];
#endif
}

/* OverWrite */
- (id)initWithFrame:(NSRect)frameRect;
{
	float width, height;
	NSImage *img;

	[super initWithFrame:frameRect];
	width = frameRect.size.width;
	height = frameRect.size.height;
	rectL.origin.x = rectL.origin.y = 0;
	rectL.size.width = width / 5.0;
	rectL.size.height = height / 9.0;
	rectP.origin.x = width * 5.0 / 6.0;
	rectP.origin.y = height * 5.0 / 6.0;
	rectP.size.width = width - rectP.origin.x;
	rectP.size.height = height -rectP.origin.y;
	if ((img = [[PrefCtrl defaultPref] currentCursor]) != nil) {
		NSPoint spot;
		spot.x = 7.0;
		spot.y = 7.0;
		[CursorR initWithImage:img hotSpot:spot];
	}
	showImage = [[NSImage alloc] initWithSize: [self frame].size];
	backImage = nil;
	return self;
}

- (void)dealloc
{
	[showImage release];
	[backImage release];
	[backgroundColor release];
	[super dealloc];
}

- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent
{
	return YES;
}

- (BOOL)becomeFirstResponder
{ /* This Method and 'makeFirstResponder:' to the window is needed to accept
     KeyDown event. */
	return YES;
}

- (void)setController:(id)obj
{
	controller = obj;
}

- (NSColor *)backgroundColor
{
	return backgroundColor;
}

- (void)setBackgroundColor:(NSColor *)clr
{
	backgroundColor = clr;
	[backgroundColor retain];
}

- (NSImage *)image
{
	return showImage;
}

- (void)drawBackground
{
	NSRect rect;
	NSPoint pt;

	[showImage lockFocus];
	if (backImage) {
		pt.x = pt.y = 0;
		[backImage compositeToPoint:pt operation:NSCompositeCopy];
	}else {
		rect = [self frame];
		rect.origin.x = rect.origin.y = -1;
		rect.size.width += 2.0;
		rect.size.height += 2.0;
		[backgroundColor set];
		PSsetlinewidth(0.0);
		NSRectFill(rect);
	}
	[showImage unlockFocus];
}


- (double)enlargeScale:(NSSize)imgsz by:(int)method err:(int *)err
{
	double xr, yr, w, ratio = 1.0;
	NSSize	vsz;

	if (err) *err = 0;
	vsz = [self frame].size;
	xr = (double)vsz.width / imgsz.width;
	yr = (double)vsz.height / imgsz.height;
	if (method == bk_FitScreen)
		ratio = (xr < yr) ? xr : yr;
	else if (err == NULL)	/* bk_CoverScreen for Bitmap */
		ratio = (xr > yr) ? xr : yr;
	else { /* bk_CoverScreen */
		if (xr > yr) {
			w = xr * imgsz.height / (double)vsz.height;
			ratio = xr;
		}else {
			w = yr * imgsz.width / (double)vsz.width;
			ratio = yr;
		}
		if (w > TooLargeRatio) { /* Too Large */
			ratio = (xr < yr) ? xr : yr; /* bk_FitScreen */
			*err = 1;
		}
	}
	return ratio;
}

- (void)makeBackground:(NSImage *)image method:(int)method
{
	NSSize sz, vsz;
	NSPoint pnt;

	if (image == nil)
		return;
	sz = [image size];
	if (sz.width <= 0 || sz.height <= 0)
		return;
	[backImage release];
	backImage = nil;
	vsz = [self frame].size;

	if (method == bk_FitScreen || method == bk_CoverScreen) {
		double ratio = [self enlargeScale:sz by:method err:NULL];
		sz.width *= ratio;
		sz.height *= ratio;
		method = bk_Centering;
		[image setScalesWhenResized:YES];
		[image setSize: sz];
	}
	if (method == bk_Centering) { /* Centering */
		[self drawBackground];	/* paint with default color */
		pnt.x = (vsz.width - sz.width) / 2;
		pnt.y = (vsz.height - sz.height) / 2;
		[showImage lockFocus];
		[image compositeToPoint:pnt operation:NSCompositeSourceOver];
		[showImage unlockFocus];
	}else {
		int i, wd, ht;
		NSRect irect;

		[showImage lockFocus];
		if (method == bk_Tiling || sz.height >= vsz.height) {
	/* Tiling */
			wd = irect.size.width = sz.width;
			ht = irect.size.height = sz.height;
		}else {
	/* Brick Work */
			wd = (int)(sz.width) / 2;
			pnt.x = -wd, pnt.y = sz.height;
			[image compositeToPoint:pnt operation:NSCompositeSourceOver];
			pnt.x = sz.width - wd;
			[image compositeToPoint:pnt operation:NSCompositeSourceOver];
			wd = irect.size.width = sz.width;
			ht = irect.size.height = sz.height * 2;
		}
		pnt.x = pnt.y = 0;
		[image compositeToPoint:pnt operation:NSCompositeSourceOver];

		irect.origin = pnt;
		for(i = wd; i < vsz.width; i += wd) {
			pnt.x = i;
			PSsplat(&irect, &pnt);
		}
		pnt.x=0;
		irect.size.width = vsz.width;
		for(i = ht; i < vsz.height; i += ht) {
			pnt.y = i;
			PSsplat(&irect, &pnt);
		}
		[showImage unlockFocus];
	}

	backImage = showImage;
	showImage = [[NSImage alloc] initWithSize: [self frame].size];
}

- (void)drawCopy:(NSImage *)img
{
	NSRect rect;
	NSPoint pt;

	rect = [self frame];
	rect.origin.x = rect.origin.y = -1;
	rect.size.width += 2.0;
	rect.size.height += 2.0;
	pt.x = pt.y = 0;
	[showImage lockFocus];
	[backgroundColor set];
	PSsetlinewidth(0.0);
	NSRectFill(rect);
	[img compositeToPoint:pt operation:NSCompositeCopy];
	[showImage unlockFocus];
}

- (void)drawRect:(NSSize)sz withColor:(NSColor *)clr andTransp:(float)trans
{
	id img = [[NSImage alloc] initWithSize:sz];
	[img lockFocus];
	[[clr colorWithAlphaComponent:trans] set];
	PSrectfill(0.0, 0.0, sz.width, sz.height);
	[img unlockFocus];
	[self drawSover:img];
	[img release];
}

- (void)drawImage:(NSImage *)img at:(NSPoint)pnt
	withBackground:(NSColor *)bgclr transparent:(float)trans
{
	id bgimg = nil;
	NSSize	sz;

	if (bgclr != nil) {
		sz = [img size];
		bgimg = [[NSImage alloc] initWithSize:sz];
		[bgimg lockFocus];
		[[bgclr colorWithAlphaComponent:trans] set];
		PSrectfill(0.0, 0.0, sz.width, sz.height);
		[bgimg unlockFocus];
	}
	[showImage lockFocus];
	if (bgimg)
		[bgimg compositeToPoint:pnt operation:NSCompositeSourceOver];
	[img compositeToPoint:pnt operation:NSCompositeSourceOver];
	[showImage unlockFocus];
	[bgimg release];
}

- (void)drawSover:(NSImage *)img
{
	NSPoint pt;
	NSSize	sz, vsz;

	sz = [img size];
	vsz = [self frame].size;
	pt.x = (vsz.width - sz.width) / 2;
	pt.y = (vsz.height - sz.height) / 2;
	[showImage lockFocus];
	[img compositeToPoint:pt operation:NSCompositeSourceOver];
	[showImage unlockFocus];
}

- (void)drawMark:(int)coloridx
{	/* kbcgmopryw */
	static unsigned char colortab[][3] = {
		{ 0, 0, 0 },
		{ 0, 0, 255 },
		{ 0, 255, 255 },
		{ 0, 255, 0 },
		{ 255, 0, 255 },
		{ 255, 127, 0 },
		{ 127, 0, 127 },
		{ 255, 0, 0 },
		{ 255, 255, 0 },
		{ 255, 255, 255 },
	};
	NSSize	sz = [self frame].size;
	float msz = sz.height / 40;
	[showImage lockFocus];
	[[NSColor blackColor] set];
	PSrectfill(sz.width - msz, sz.height - msz - 4, msz, msz);
	[[NSColor colorWithRGBElements:colortab[coloridx]] set];
	PSrectfill(sz.width - msz - 2, sz.height - msz - 2, msz, msz);
	[showImage unlockFocus];
}


- (void)resetCursorRects
{
	[self addCursorRect:[self frame] cursor:CursorR];
	[self addCursorRect:rectL cursor:CursorL];
	[self addCursorRect:rectP cursor:CursorP];
}

- (void)mouseDown:(NSEvent *)event
{
	NSPoint	p;
	int	dmy;

	if ([event type] != NSLeftMouseDown)
		return;
	dmy = [event clickCount];	/* prevent error multi-clicking */
	p = [event locationInWindow];
	p = [self convertPoint:p fromView:nil]; /* View based point */
	if (NSMouseInRect(p, rectL, NO))
		[controller prevPush:self];
	else if (NSMouseInRect(p, rectP, NO))
		[controller quitPush:self];
	else
		[controller nextPush:self];
}

- (void)keyDown:(NSEvent *)event
{
	unichar ev = [[event characters] characterAtIndex:0];
	switch (ev) {
	case 'f': case 'n':
	case 'F': case 'N':
	case ' ': case '\n': case '\t':
	case NSRightArrowFunctionKey:
		[controller nextPush:self];
		break;
	case 'b': case 'p':
	case 'B': case 'P':
	case '\010' /* BS */: case '\177' /* DEL */:
	case NSLeftArrowFunctionKey:
		[controller prevPush:self];
		break;
	case 'q': case 'Q':
	case '\033' /* ESC */:
		[controller quitPush:self];
		break;
	default:
		break;
	}
}

- (void)drawRect:(NSRect)rect
{
	[showImage compositeToPoint:rect.origin
		fromRect:rect operation:NSCompositeCopy];
}

@end
