#import "InspectorCtrl.h"
#import <Foundation/NSBundle.h>
#import <Foundation/NSNotification.h>
#import <AppKit/NSPanel.h>
#import "ScenarioSource.h"
#import "SDTableView.h"

#define  InspectorPanelName	@"Inspector"

static InspectorCtrl *defaultController = nil;

@implementation InspectorCtrl

+ (id)defaultController
{
	if (defaultController == nil)
		defaultController = [[self alloc] init];
	return defaultController;
}

+ (void)appShouldTerminate
{
	if (defaultController == nil)
		return;
	[defaultController->panel performClose:self];
}

- (void)loadNib
{
	id	notify;

	if (panel != nil)
		return;
	[NSBundle loadNibNamed:@"Inspector.nib" owner:self];
	[panel setDelegate:self];
	[panel setFrameUsingName: InspectorPanelName];
	notify = [NSNotificationCenter defaultCenter];
	[notify addObserver:self
		selector:@selector(selectionChanged:)
		name:NSTableViewSelectionDidChangeNotification object:nil];
	[notify addObserver:self
		selector:@selector(selectionChanged:)
		name:ScenarioDidChange object:nil];
}

- (void)activateAndOrderFront:(id)sender
{
	[self loadNib];
	[panel makeKeyAndOrderFront:self];
	isOpened = YES;
}

/* Local Method */
- (void)selectNothing
{
	[optext setStringValue:@""];
	[displaySW setEnabled:NO];
	[imageSzPM setEnabled:NO];
	[captionRB setEnabled:NO];
	[backImageSW setEnabled:NO];
	[backImagePM setEnabled:NO];
	[noTextColorSW setEnabled:NO];
	[textTranspSW setEnabled:NO];
	[sheetMarkPM setEnabled:NO];
	[baselineSL setEnabled:NO];
	[baselineSW setEnabled:NO];
	[fontsizeSL setEnabled:NO];
	[fontsizeSW setEnabled:NO];
}

static int info2idx(int n)
{
	if (n == im_Auto) return 0;
	if (n < im_Integer)
		return (n + 3);
	if (n == im_Integer) return 1;
	return (n == im_Screen) ? 2 : 3;
}

/* Local Method */
- (void)setStateAsOption
{
	[backImageSW setEnabled:YES];
	if (optionVal.backgflag) {
		[backImageSW setState:YES];
		[backImagePM setEnabled:YES];
		[backImagePM selectItemAtIndex: optionVal.background];
		[displaySW setEnabled:NO];
		[imageSzPM setEnabled:NO];
		[captionRB setEnabled:NO];
		[noTextColorSW setEnabled:NO];
		[textTranspSW setEnabled:NO];
		[sheetMarkPM setEnabled:NO];
		[baselineSL setEnabled:NO];
		[baselineSW setEnabled:NO];
		[fontsizeSL setEnabled:NO];
		[fontsizeSW setEnabled:NO];
		return;
	}
	[backImageSW setState:NO];
	[backImagePM setEnabled:NO];
	[displaySW setEnabled:YES];
	[displaySW setState: optionVal.overlap];
	[imageSzPM setEnabled:YES];
	[imageSzPM selectItemAtIndex: info2idx(optionVal.imageSz)];
	[captionRB setEnabled:YES];
	[captionRB selectCellWithTag: optionVal.title];
	[noTextColorSW setEnabled:YES];
	[noTextColorSW setState: optionVal.notextclr];
	[textTranspSW setEnabled:YES];
	[textTranspSW setState: optionVal.texttransp];
	[sheetMarkPM setEnabled:YES];
	[sheetMarkPM selectItemAtIndex:
		(optionVal.mark ? ((optionVal.mark & 0x0f) + 1) : 0)];

	[baselineSW setEnabled:YES];
	if (optionVal.baseline == 0) {
		[baselineSW setState:NO];
		[baselineSL setEnabled:NO];
	}else {
		[baselineSW setState:YES];
		[baselineSL setEnabled:YES];
		[baselineSL setIntValue: optionVal.baseline];
		[baselineTX setStringValue:
		    [NSString stringWithFormat:@"+%d", optionVal.baseline]];
	}

	[fontsizeSW setEnabled:YES];
	if (optionVal.fontsize == 0) {
		[fontsizeSW setState:NO];
		[fontsizeSL setEnabled:NO];
	}else {
		[fontsizeSW setState:YES];
		[fontsizeSL setEnabled:YES];
		[fontsizeSL setIntValue: optionVal.fontsize];
		[fontsizeTX setStringValue:
		    [NSString stringWithFormat:@"%d", optionVal.fontsize]];
	}
}

/* Local Method */
- (void)setAttributes
{
	int	row;
	NSString *opt;

	if (!isOpened)
		return;
	if (currentTable == nil || [currentTable numberOfSelectedRows] != 1) {
		[self selectNothing];
		return;
	}
	row = [currentTable selectedRow];
	opt = [[currentTable dataSource] optionStringAtIndex:row];
	if (opt == nil) {
		opt = @"";
		clearOption(&optionVal);
	}else
		convStringToOption(opt, &optionVal);
	[optext setStringValue:opt];
	[self setStateAsOption];
	[panel display];
}

- (void)setCurrentTable:(id)table
{
	currentTable = table;
	[self setAttributes];
}

- (void)selectionChanged:(NSNotification *)aNotification
{
	id	obj = [aNotification object];
	if ([obj isKindOfClass:[NSTableView class]])
		[self setCurrentTable:obj];
	else /* Notification from ScenarioSource, perhaps */
		[self setAttributes];
}

/* Local Method */
- (void)setOptionString:(NSString *)str
{
	int row;
	[optext setStringValue: str];
	row = [currentTable selectedRow];
	if ([[currentTable dataSource] setOptionString:str atIndex:row])
		[currentTable reloadData];
}

- (void)changeDisplay:(id)sender
{
	optionVal.overlap = [displaySW state];
	[self setOptionString: convToString(&optionVal)];
}

- (void)changeImageSise:(id)sender
{
	int n = [imageSzPM indexOfSelectedItem];
	switch (n) {
	case 0: /* Auto */
		optionVal.imageSz = im_Auto;
		break;
	case 1: /* Integer */
		optionVal.imageSz = im_Integer;
		break;
	case 2: case 3:
		optionVal.imageSz = im_Screen + (n - 2);
		break;
	default:
		optionVal.imageSz = n - 3;
		break;
	}
	[self setOptionString: convToString(&optionVal)];
}

- (void)changeCaption:(id)sender
{
	optionVal.title = [captionRB selectedTag];
	[self setOptionString: convToString(&optionVal)];
}

- (void)changeBackImage:(id)sender
{
	optionVal.background = [backImagePM indexOfSelectedItem];
	[self setOptionString: convToString(&optionVal)];
}

- (void)toggleBackImage:(id)sender
{
	optionVal.backgflag = [backImageSW state];
	[self setStateAsOption];
	[self setOptionString: convToString(&optionVal)];
}

- (void)toggleTextColor:(id)sender
{
	optionVal.notextclr = [noTextColorSW state];
	[self setOptionString: convToString(&optionVal)];
}

- (void)toggleTextTransp:(id)sender
{
	optionVal.texttransp = [textTranspSW state];
	[self setOptionString: convToString(&optionVal)];
}

- (void)changeSheetMark:(id)sender
{
	int tag = [sheetMarkPM indexOfSelectedItem];
	optionVal.mark = tag ? ((tag - 1) | 0x10) : 0;
	[self setOptionString: convToString(&optionVal)];
}

- (void)changeBaseline:(id)sender
{
	optionVal.baseline = [baselineSL intValue];
	[baselineTX setStringValue:
		[NSString stringWithFormat:@"+%d", optionVal.baseline]];
	[self setOptionString: convToString(&optionVal)];
}

- (void)toggleBaseline:(id)sender
{
	if ([baselineSW state]) {
		[baselineSL setEnabled:YES];
		[self changeBaseline:self];
	}else {
		[baselineSL setEnabled:NO];
		optionVal.baseline = 0;
		[self setOptionString: convToString(&optionVal)];
	}
}

- (void)changeFontSize:(id)sender
{
	optionVal.fontsize = [fontsizeSL intValue];
	[fontsizeTX setStringValue:
		[NSString stringWithFormat:@"%d", optionVal.fontsize]];
	[self setOptionString: convToString(&optionVal)];
}

- (void)toggleFontSize:(id)sender
{
	if ([fontsizeSW state]) {
		[fontsizeSL setEnabled:YES];
		[self changeFontSize:self];
	}else {
		[fontsizeSL setEnabled:NO];
		optionVal.fontsize = 0;
		[self setOptionString: convToString(&optionVal)];
	}
}

/* NSWinodow's Delegate */
- (BOOL)windowShouldClose:(id)sender
{
	isOpened = NO;
	[panel saveFrameUsingName: InspectorPanelName];
	return YES;
}

@end
