#import "PrefCtrl.h"
#import <Foundation/NSBundle.h>
#import <AppKit/NSScreen.h>
#import <AppKit/NSColorWell.h>
#import <AppKit/NSPanel.h>
#import <Foundation/NSUserDefaults.h>
#import <stdio.h>
#import "NSColorAdditional.h"
#import "SettingDictionary.h"
#import "dickeys.h"

#define  ScreenSizeTAG	@"ScreenSizeTag"
#define  DottedFile	@"DottedFile"
#define  MainBodyRatio	@"MainBodyRatio"
#define  SelectedCursor	@"SelectedCursor"
#define  BodyRatioMin	0.5
#define  BodyRatioDef	0.85
#define  PrefPanelName	@"Preferences"

static id defaultPref = nil;
static NSSize screenSize;
static short scSize[][2] = {
	{ 0, 0 },
	{ 640, 480 },
	{ 800, 600 },
	{ 1024, 768 },
	{ 1152, 864 },
	{ 1280, 1024 },
	{ 1600, 1200 },
	{ 1920, 1440 },
	{ 2048, 1536 },
	{ 0, 0 }
};
/* Keys used in setting info. */
static NSString *arraykeys[] = {
	SBackgColor, STextColor, STextBgColor, nil
};
static NSString *strkeys[] = {
	STransparent, SUseTextBg, SEPSBackg, SBaseline, SFontSize, STextFont,
	nil
};

@implementation PrefCtrl

+ (id)defaultPref
{
	if (defaultPref == nil) {
		defaultPref = [[self alloc] init];
		screenSize = [[NSScreen mainScreen] frame].size;
	}
	return defaultPref;
}

+ (void)appShouldTerminate
{
	[defaultPref saveDefaults];
	[defaultPref release];
	defaultPref = nil;
}

- (id)init
{
	id	usrdef, dic, ar;
	int	i;

	[super init];
	changeFlag = NO;
	dic = [NSMutableDictionary dictionaryWithCapacity:5];
	usrdef = [NSUserDefaults standardUserDefaults];
	sizeTag = [usrdef integerForKey:ScreenSizeTAG];
	/* Note: Without the entry for the key, this method returns 0. */
	dotted = [usrdef boolForKey:DottedFile];
	if ((bodyRatio = [usrdef floatForKey:MainBodyRatio]) < BodyRatioMin)
		bodyRatio = BodyRatioDef;
	else if (bodyRatio > 1.0)
		bodyRatio = 1.0;
	cursorTag = [usrdef integerForKey:SelectedCursor];
	/* Note: Without the entry for the key, this method returns 0. */
	for (i = 0; arraykeys[i]; i++) {
		if ((ar = [usrdef stringArrayForKey:arraykeys[i]]) != nil
				&& [ar isKindOfClass:[NSArray class]])
			[dic setObject:ar forKey:arraykeys[i]];
		/* If ar is not an array, */
		/* perhaps, we fetch global setting for the keys. Ignore. */
	}
	for (i = 0; strkeys[i]; i++) {
		if ((ar = [usrdef stringForKey:strkeys[i]]) != nil)
			[dic setObject:ar forKey:strkeys[i]];
	}
	defaultDic = [[SettingDictionary alloc] initWithDictionary:dic];
	return self;
}

- (void)dealloc
{
	[defaultDic release];
	[panel close];
	[super dealloc];
}

- (void)saveDefaults
{
	id	usrdef, obj;
	int	i;

	if (!changeFlag)
		return;	/* Default values are not changed */

	usrdef = [NSUserDefaults standardUserDefaults];
	[usrdef setInteger:sizeTag forKey:ScreenSizeTAG];
	[usrdef setBool:dotted forKey:DottedFile];
	[usrdef setFloat:bodyRatio forKey:MainBodyRatio];
	[usrdef setInteger:cursorTag forKey:SelectedCursor];

	for (i = 0; arraykeys[i]; i++) {
		obj = [defaultDic objectForKey:arraykeys[i]];
		[usrdef setObject:obj forKey:arraykeys[i]];
	}
	for (i = 0; strkeys[i]; i++) {
		obj = [defaultDic objectForKey:strkeys[i]];
		[usrdef setObject:obj forKey:strkeys[i]];
	}

	[panel saveFrameUsingName: PrefPanelName];
	changeFlag = NO;
}

/* Local Method */
- (void)loadNib
{
	int	i;

	if (panel != nil)
		return;
	[NSBundle loadNibNamed:@"Preferences.nib" owner:self];
	// [panel setDelegate:self];
	for (i = 1; scSize[i][0]; i++) {
		if (scSize[i][0] > screenSize.width
		|| scSize[i][1] > screenSize.height) break;
	}
	if (sizeTag >= i)
		sizeTag = 0;
	[screenSizeMenu selectItemAtIndex: sizeTag];
	[dottedFileSW setState:dotted];
	[bodyRatioSL setFloatValue:bodyRatio];
	[bodyRatioTX setFloatValue:bodyRatio];
	[cursorRB selectCellWithTag:cursorTag];
	for ( ; scSize[i][0]; i++)
		[[screenSizeMenu itemAtIndex:i] setEnabled:NO];
	[panel setFrameUsingName: PrefPanelName];
}

- (void)makeKeyAndOrderFront:(id)sender
{
	[self loadNib];
	[panel makeKeyAndOrderFront: sender];
}

- (void)screenSizeChanged:(id)sender
{
	changeFlag = YES;
	sizeTag = [screenSizeMenu indexOfSelectedItem];
}

- (NSSize)screenSize
{
	NSSize sz;

	if (sizeTag == 0)
		return screenSize;
	sz.width = scSize[sizeTag][0];
	sz.height = scSize[sizeTag][1];
	return sz;
}

- (void)toggleDottedFile:(id)sender
{
	changeFlag = YES;
	dotted = [dottedFileSW state];
}

- (BOOL)isDottedFileListed
{
	return dotted;
}

- (void)bodyRatioChanged:(id)sender
{
	changeFlag = YES;
	bodyRatio = [bodyRatioSL floatValue];
	bodyRatio = (int)(bodyRatio * 100 + 0.5) / 100.0;
	[bodyRatioTX setFloatValue:bodyRatio];
}

- (float)bodyRatio
{
	return bodyRatio;
}

- (void)cursorChanged:(id)sender
{
	int tag = [cursorRB selectedTag];
	if (tag != cursorTag) {
		changeFlag = YES;
		cursorTag = tag;
	}
}

- (NSImage *)currentCursor
{
	[self loadNib];
	return [[cursorRB selectedCell] image];
}

- (void)setAsDefaultValue:(struct setinfo *)info
{
	[defaultDic release];
	defaultDic = [[SettingDictionary alloc] initWithSettingInfo:info];
	changeFlag = YES;
	[self saveDefaults];
}

- (struct setinfo *)defaultSetting
{
	return [defaultDic info];
}

@end
