#import "ScenarioCtrl.h"
#import <Foundation/NSString.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSSet.h>
#import <Foundation/NSBundle.h>
#import <Foundation/NSFileManager.h>
#import <Foundation/NSNotification.h>
#import <AppKit/NSPanel.h>	/* NSRunAlertPanel() */
#import <AppKit/NSScreen.h>
#import <AppKit/NSEvent.h>
#import <AppKit/NSImage.h>
#import <AppKit/NSWorkspace.h>
#import "ScenarioSource.h"
#import "SDTableView.h"
#import "ToyProjCtrl.h"
#import "PrefCtrl.h"
#import "InspectorCtrl.h"
#import "SlideShowCtrl.h"
#import "FileSelector.h"
#import "SettingCtrl.h"
#import "commtypes.h"

#define  docImageName	@"IconDoc"
#define  emptyScenario  @"{ Scenario = (); Setting = {}; }\n"

@implementation ScenarioCtrl

static NSSize screenSize;
static NSImage *docImage;

+ (void)initialize
{
	NSString *s;

	screenSize = [[NSScreen mainScreen] frame].size;
	s = [[NSBundle mainBundle] pathForResource:docImageName ofType:@"tiff"];
	docImage = [[NSImage alloc] initWithContentsOfFile:s];
}

/* Local Method */
- (void)locateWindow
{
	NSPoint pnt;
	static int winCount = 0;

	++winCount;
	if (screenSize.width <= 640) {
		pnt.x = (winCount % 5) * 20 + 150;
		pnt.y = (winCount % 4) * 20 + 30;
	}else {
		pnt.x = (winCount % 7) * 20 + 200;
		pnt.y = (winCount % 6) * 20 + 40;
	}
	pnt.y = screenSize.height - pnt.y;
	[window setFrameTopLeftPoint: pnt];
}

- (void)loadNib
{
	id	sc;
	id	newtable;
	id	cl;
	int	n, i;

	[NSBundle loadNibNamed:@"Scenario.nib" owner:self];
	source = [[ScenarioSource alloc] init];
	[source setController:self];
	//
	// Replace NSTableView with SDTableView !!
	//
	sc = [[table superview] superview];	/* NSScrollView */
	newtable = [[SDTableView alloc] init];
	[newtable setHasHeader: YES];
	cl = [table tableColumns];
	n = [cl count];
	for (i = 0; i < n; i++)
		[newtable addTableColumn:[cl objectAtIndex:i]];
	[sc setDocumentView: newtable];
	table = newtable;
	[table tile];
	[table setDataSource: source];
	[table setDelegate: source];
	[table setTarget: source];
	[table setDoubleAction:@selector(doubleClicked:)];
	[table setAllowsMultipleSelection:YES];
	[table setConstrainedMove:YES];
	[table deselectAll:self];
	[[NSNotificationCenter defaultCenter] addObserver:self
		selector:@selector(tableDidChange:)
		name:nil object:source];

	[self locateWindow];
	[window setTitleWithRepresentedFilename: filepath];
	[window setMiniwindowImage:docImage];
	[window makeKeyAndOrderFront:self];
}

/* Local Method */
- (void)removePlaceholder
{
	if (fileDeletable) {
		NSFileManager *manager = [NSFileManager defaultManager];
		(void)[manager removeFileAtPath:filepath handler:nil];
		fileDeletable = NO;
	}
}

- (void)dealloc
{
	[self removePlaceholder];
	[filepath release];
	[selector release];
	if (setting)
		freeSetinfo(setting);
	[settingCtrl release];
	[source release];
	[super dealloc];
}

- (BOOL)setAndCreateDir:(NSString *)fn
{
	NSFileManager *manager;
	NSString *msg;
	BOOL	isdir;

	if ([[fn pathExtension] isEqualToString:EXTENSION])
		filepath = fn;
	else
		filepath = [fn stringByAppendingPathExtension:EXTENSION];
	[filepath retain];
	manager = [NSFileManager defaultManager];

	/* Being a file with the same name, remove it. */
	if ([manager fileExistsAtPath:filepath isDirectory:&isdir]) {
		int tag;
		id ws = [NSWorkspace sharedWorkspace];
		id fs = [NSArray arrayWithObject:[filepath lastPathComponent]];
		if(! [ws performFileOperation:NSWorkspaceRecycleOperation
			source:[filepath stringByDeletingLastPathComponent]
			destination:@"" files:fs tag:&tag] ) {
			msg = NSLocalizedString(@"Can not replace", Err_Replace);
			NSRunAlertPanel(NSLocalizedString(@"Error", Error),
				@"%@ :\n%@", @"", nil, nil, msg, filepath);
			return NO;
		}
	}

	/* Create new file as a placeholder */
	if (![emptyScenario writeToFile:filepath atomically:NO]) {
		msg = NSLocalizedString(@"Can not create file", Err_CreateFile);
		NSRunAlertPanel(NSLocalizedString(@"Error", Error),
			@"%@ :\n%@", @"", nil, nil, msg, filepath);
		return NO;
	}
	[self loadNib];
	fileDeletable = YES;
	setting = copySetinfo([[PrefCtrl defaultPref] defaultSetting]);
	slideshow = nil;
	[window setDocumentEdited:YES];
	return YES;
}

- (BOOL)setAndOpenDoc:(NSString *)fn
{
	id	dic;

	filepath = fn;
	[filepath retain];
	[self loadNib];
	fileDeletable = NO;
	if ((dic = [source readFromFile:filepath]) == nil)
		return NO;
	/* This call returns setting info. from SettingDictionary */
	setting = copySetinfo([dic info]);
	slideshow = nil;
	[table reloadData];
	[window setDocumentEdited:NO];
	return YES;
}

- (void)save
{
	[self saveScenario: self];
}

- (void)saveIfNeeded
{
	if ([window isDocumentEdited])
		[self save];
}

- (void)saveAs:(NSString *)newname
{
	id	darr, farr;
	NSString *diff;
	int	n, i;

	darr = [[[newname stringByStandardizingPath]
		stringByDeletingLastPathComponent] pathComponents];
	farr = [[[filepath stringByStandardizingPath]
		stringByDeletingLastPathComponent] pathComponents];
#if 0
	n = [darr count];
	if (n > [farr count])
		return ERROR;
	for (i = 0; i < n; i++)
		if (![[darr objectAtIndex:i]
			isEqualToString:[darr objectAtIndex:i]])
			return ERROR;
#endif
	[self removePlaceholder];
	[settingCtrl closePanel:self];
	/* new setting will be newly set by method activateSettingCtrl: */
	if (selector) {
		[selector release];
		selector = nil;
	}
	if ((i = [darr count]) != (n = [farr count])) {
		/* save in an ancestor directory */
		diff = [farr objectAtIndex:i];
		for (++i; i < n; i++)
			diff = [diff stringByAppendingPathComponent:
						[farr objectAtIndex:i]];
		[source insertDirectoryElement:diff];
		[table reloadData];
	}
	[filepath release];
	[(filepath = newname) retain];
	[window setTitleWithRepresentedFilename: filepath];
	[self saveScenario: self];
}

/* Editing Action Method: because sometimes the table is not active */
- (void)delete:(id)sender
{
	[table delete:sender];
}

- (void)reload
{
	id	dic;
	int	ans;

	ans = NSRunAlertPanel(NSLocalizedString(@"Warning", Warning),
		NSLocalizedString(@"Really Reload", ReallyReload),
		NSLocalizedString(@"Do Reload", DoReload),
		NSLocalizedString(@"Cancel", Cancel), nil);
	if (ans != NSAlertDefaultReturn)
		return;
	if ((dic = [source readFromFile:filepath]) == nil)
		return;
	/* This call returns object(autorelease) of SettingDictionary */
	freeSetinfo(setting);
	setting = copySetinfo([dic info]);

	[settingCtrl closePanel:self];
	/* new setting will be newly set by method activateSettingCtrl: */
	[table reloadData];
	[table deselectAll:self];
	[window setDocumentEdited:NO];
}

- (void)tableDidChange:(NSNotification *)notification
{
	[window setDocumentEdited:YES];
}

- (void)settingChanged:(id)sender
{
	[window setDocumentEdited:YES];
}

- (void)addFiles:(NSArray *)flist
{
	int row;
	NSString *root;

	if ((row = [table selectedRow]) >= 0)
		++row;
	else /* not selected */
		row = [source count];
	root = [filepath stringByDeletingLastPathComponent];
	row += [source addFiles:flist inDir:root fromIndex:row];
	[table reloadData];
	[table selectRow:(row-1) byExtendingSelection:NO];
}

- (void)openFileByWS:(NSString *)fn
{
	NSString *path = [filepath stringByDeletingLastPathComponent];
	[[NSWorkspace sharedWorkspace] openFile:
		[path stringByAppendingPathComponent:fn]];
}

- (NSString *)filename
{
	return filepath;
}

- (id)window
{
	return window;
}

- (void)makeKeyAndOrderFront:(id)sender
{
	[window makeKeyAndOrderFront:sender];
}

- (void)setController:(id)obj
{
	controller = obj;
}

- (void)activateInspector:(id)sender
{
	id inspector = [InspectorCtrl defaultController];
	[inspector activateAndOrderFront:self];
	[inspector setCurrentTable: table];
}

- (void)activateSelector:(id)sender
{
	static Class fileselector = Nil;

	if (selector == nil) {
		if (!fileselector) {	/* Load Class FileSelector */
			NSBundle *bundle = [NSBundle mainBundle];
			NSString *path = [bundle pathForResource:@"FileSelect"
					ofType:@"bundle"];
			bundle = [NSBundle bundleWithPath: path];
			if (bundle == nil) /* ERROR */
				return;
			fileselector = [bundle classNamed:@"FileSelector"];
		}
		selector = [[fileselector alloc] init];
		[selector setPath:
			[filepath stringByDeletingLastPathComponent]];
		[selector setScenarioCtrl:self];
	}
	[selector activateAndOrderFront:self];
}

- (void)activateSettingCtrl:(id)sender
{
	static Class settingController = Nil;

	if (settingCtrl == nil) {
		if (!settingController) { /* Load Class SettingCtrl */
			NSBundle *bundle = [NSBundle mainBundle];
			NSString *path = [bundle pathForResource:@"Setting"
					ofType:@"bundle"];
			bundle = [NSBundle bundleWithPath: path];
			if (bundle == nil) /* ERROR */
				return;
			settingController = [bundle classNamed:@"SettingCtrl"];
		}
		settingCtrl = [[settingController alloc] init];
		[settingCtrl setScenarioCtrl:self];
	}
	[settingCtrl setSetting:setting];
	[settingCtrl activateAndOrderFront:self];
}

- (void)checkFileList:(id)sender
{
	int	num;

	if ((num = [source checkFileExistence]) < 0) {
		NSBeep();
		return;
	}
	if (num == 0) {
		NSRunAlertPanel(@"",
			NSLocalizedString(@"All files exist", Chk_OK),
			@"", nil, nil);
		return;
	}
	[table reloadData];
	NSRunAlertPanel(
		NSLocalizedString(@"Warning", Warning),
		NSLocalizedString(@"%d files do not exist", Chk_NG),
			@"", nil, nil, num);
}

- (void)saveScenario:(id)sender
{
	id dic = [[SettingDictionary alloc] initWithSettingInfo:setting];
	if ([source writeToFile:filepath with:[dic dictionary]]) {
		[window setDocumentEdited:NO];
		fileDeletable = NO;
	}
	[dic release];
}

/* Local Method */
- (void)showDidQuit
{
	[slideshow release];
	slideshow = nil;
}

- (void)start:(id)sender
{
	int row;

	if (slideshow != nil)
		return;	/* not to activate slideshow more than one */
	if ([sender tag] == 0)
		row = 0;
	else if ((row = [table selectedRow]) < 0) {
		NSBeep();
		return;
	}
	slideshow = [[SlideShowCtrl alloc]
			initWithTable:table andSetting:setting];
	[slideshow setOHPPath: [filepath stringByDeletingLastPathComponent]];
	[slideshow setQuitAction:@selector(showDidQuit) to:self];
	[slideshow startFrom: row];
}


/* Local Method */
- (unsigned int)askSaveAndClose:(BOOL)flag
{
	if (![window isDocumentEdited])
		return NSAlertAlternateReturn;	/* Don't Save */
	return NSRunAlertPanel(
		NSLocalizedString(@"Close", Close),
		NSLocalizedString(@"Save changes to %@ ?", SaveChanges),
		NSLocalizedString(@"Save", Save),
		NSLocalizedString(@"Don't Save", NotSave),
		(flag ? nil : NSLocalizedString(@"Cancel", Cancel)),
		[filepath lastPathComponent]
	);
}

- (void)quitShow
{
	if (slideshow)
		[slideshow quitPush:self];	/* Quit SlideShow */
}

- (void)appShouldQuit
{
	if ([window isDocumentEdited]) {
		[window makeKeyAndOrderFront:self];
		if ([self askSaveAndClose:YES] == NSAlertDefaultReturn)
			[self saveScenario: self];
	}
	// Never do [controller scenarioShouldClose: self]; !!!
	[window setDelegate: nil];
	[window performClose:self];
}

/* NSWinodow's Delegate */
- (BOOL)windowShouldClose:(id)sender
{
	if ([window isDocumentEdited]) {
		int r = [self askSaveAndClose:NO];
		switch (r) {
		case NSAlertDefaultReturn:	/* Save */
			[self saveScenario: self];
			break;
		case NSAlertAlternateReturn:	/* Don't Save */
			break;
		case NSAlertOtherReturn:	/* Cancel */
		default:
			return NO;
		}
	}
	[controller scenarioShouldClose: self];
	[window setDelegate: nil];
	[self release];
	return YES;
}

/* NSWinodow's Delegate */
- (void)windowDidBecomeMain:(NSNotification *)aNotification
{
	id inspector = [InspectorCtrl defaultController];
	[inspector setCurrentTable: table];
}

@end
