#import "ScenarioSource.h"
#import <Foundation/NSString.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSValue.h>
#import <stdlib.h>
#import <libc.h>
#import "commtypes.h"

@implementation ScenarioSource (ActionMethods)

enum {
	op_copy, op_cut, op_delete, op_paste
};

static NSMutableArray *mypboard = nil;

/* Local Method */
- (BOOL)do:(int)op selectedRow:(id)sender
{
	int	i, num;
	unsigned int *narr;
	NSEnumerator *en;
	id	obj;

	if ((num = [sender numberOfSelectedRows]) <= 0) {
		NSBeep();
		return NO;
	}
	narr = (unsigned int *)malloc(sizeof(unsigned int) * (num + 1));
	en = [sender selectedRowEnumerator];
	for (i = 0; (obj = [en nextObject]) != nil; i++)
		narr[i] = [obj unsignedIntValue];
	if (op == op_copy || op == op_cut) {
		if (mypboard) [mypboard release];
		mypboard = [[NSMutableArray alloc] initWithCapacity:num];
		for (i = 0; i < num; i++)
			[mypboard addObject:[records objectAtIndex:narr[i]]];
	}
	if (op == op_cut || op == op_delete) {
		[records removeObjectsFromIndices:narr numIndices:num];
		if (num == 1) {	/* select */
			if ((i = narr[0]) >= [records count])
				i = [records count] - 1;
			[sender selectRow:i byExtendingSelection:NO];
		}else
			[sender deselectAll:self];
	}

	free((void *)narr);
	return YES;
}

/* Delegate for Editing */
- (void)copy:(id)sender
{
	[self do:op_copy selectedRow: sender];
}

- (void)cut:(id)sender
{
	if ([self do:op_cut selectedRow: sender])
		[sender reloadData];
	[self castNotification];
}

- (void)delete:(id)sender
{
	if ([self do:op_delete selectedRow: sender])
		[sender reloadData];
	[self castNotification];
}

- (void)paste:(id)sender
{
	int	i, row, num;

	if (mypboard == nil || (num = [mypboard count]) <= 0)
		return;
	if ([sender numberOfSelectedRows] != 1) {
		NSBeep();
		return;
	}
	row = [sender selectedRow];
	for (i = 0; i < num; i++) {
		id obj = [[mypboard objectAtIndex:i] mutableCopy];
		[records insertObject: obj atIndex:++row];
		[obj release];	// obj is retain'ed by records
	}
	if (num == 1)
		[sender selectRow:row byExtendingSelection:NO];
	[sender reloadData];
	[self castNotification];
}

@end
