#import "../SettingCtrl.h"
#import <Foundation/NSArray.h>
#import <Foundation/NSString.h>
#import <Foundation/NSBundle.h>
#import <AppKit/NSPanel.h>
#import <AppKit/NSColorWell.h>
#import <AppKit/NSFont.h>
#import <AppKit/NSFontManager.h>
#import <AppKit/NSFontPanel.h>
#import "../NSColorAdditional.h"
#import "ColorSampleView.h"
#import "../PrefCtrl.h"
#import "../ScenarioCtrl.h"
#import "../dickeys.h"

#define  Bias	18
static int serNo = 0;
static NSPoint defaultOrigin;

@implementation SettingCtrl

- (void)dealloc
{
	[panel setDelegate:nil];
	[panel close];
	[panel release];
	[super dealloc];
}

/* Local Method */
- (void)loadNib
{
	if (panel == nil) {
		[NSBundle loadNibNamed:@"Setting.nib" owner:self];
		if (serNo++ == 0)
			defaultOrigin = [panel frame].origin;
		else {
			NSPoint pt;
			pt.x = defaultOrigin.x + (serNo % 3) * Bias;
			pt.y = defaultOrigin.y + (serNo % 4) * Bias;
			[panel setFrameOrigin:pt];
		}
	}
}

- (void)activateAndOrderFront:(id)sender
{
	id title;

	[self loadNib];
	changeFlag = NO;
	[backgColorWell setColor:setting->bgcolor];
	[sampleView setBackgColor:setting->bgcolor];
	[textColorWell setColor:setting->txcolor];
	[sampleView setTextColor:setting->txcolor];
	[textBgColorWell setColor:setting->txbgcolor];
	[baselineTX setStringValue:
		[NSString stringWithFormat:@"+%d", setting->baseline]];
	[baselineSL setIntValue:setting->baseline];
	[fontsizeTX setStringValue:
		[NSString stringWithFormat:@"%d", setting->fontsize]];
	[fontsizeSL setIntValue:setting->fontsize];
	if (setting->usetxbg) {
		[textBgSW setState:YES];
		[textBgColorWell setEnabled:YES];
		[textTransp setEnabled:YES];
		[sampleView setTextBgColor:setting->txbgcolor
				andTransparent:setting->transparent];
	}else {
		[textBgSW setState:NO];
		[textBgColorWell setEnabled:NO];
		[textTransp setEnabled:NO];
		[sampleView setTextBgColor:nil andTransparent:1.0];
	}
	[fontnameTX setStringValue:[setting->txfont fontName]];
	[EPSMenu selectItemAtIndex:setting->epsbg];
	[revertButton setEnabled:changeFlag];
	[panel setDocumentEdited:NO];
	title = [NSString stringWithFormat:@"%@ : %@",
		NSLocalizedString(@"Setting", Setting),
		[[scenarioCtrl filename] lastPathComponent]];
	[panel setTitle:title];
	[panel makeKeyAndOrderFront:self];
	[sampleView display];
}

- (void)setScenarioCtrl:(id)scenario
{
	scenarioCtrl = scenario;
}

- (void)closePanel:(id)sender
{
	[panel close];
}

- (void)setSetting:(struct setinfo *)newsetting
	/* This method should be called before "activateAndOrderFront:" */
{
	setting = newsetting;	/* just refering, no need to release */
	changeFlag = NO;
	[revertButton setEnabled:changeFlag];
	[panel setDocumentEdited:NO];
}

- (void)changeWell:(id)sender
{
	if ([sender tag] == 0)
		[sampleView setBackgColor:[backgColorWell color]];
	else
		[sampleView setTextColor:[textColorWell color]];
	[sampleView display];
	changeFlag = YES;
	[revertButton setEnabled:changeFlag];
	[panel setDocumentEdited:YES];
}

- (void)changeTextBgSwitch:(id)sender
{
	if ([textBgSW state]) {
		[textBgColorWell setEnabled:YES];
		[textTransp setEnabled:YES];
		[sampleView setTextBgColor:[textBgColorWell color]
			andTransparent:[textTransp floatValue]];
	}else {
		[textBgColorWell setEnabled:NO];
		[textTransp setEnabled:NO];
		[sampleView setTextBgColor:nil andTransparent:1.0];
	}
	[sampleView display];
	changeFlag = YES;
	[revertButton setEnabled:changeFlag];
	[panel setDocumentEdited:YES];
}

- (void)changeTextBgParameters:(id)sender
{
	[sampleView setTextBgColor:[textBgColorWell color]
			andTransparent:[textTransp floatValue]];
	[sampleView display];
	changeFlag = YES;
	[revertButton setEnabled:changeFlag];
	[panel setDocumentEdited:YES];
}

- (void)changeBaseline:(id)sender
{
	int val = [baselineSL intValue];
	[baselineTX setStringValue:[NSString stringWithFormat:@"+%d", val]];
	changeFlag = YES;
	[revertButton setEnabled:changeFlag];
	[panel setDocumentEdited:YES];
}

- (void)changeFontSize:(id)sender
{
	int val = [fontsizeSL intValue];
	[fontsizeTX setStringValue:[NSString stringWithFormat:@"%d", val]];
	changeFlag = YES;
	[revertButton setEnabled:changeFlag];
	[panel setDocumentEdited:YES];
}

- (void)changeEPSSetting:(id)sender
{
	int tag = [EPSMenu indexOfSelectedItem];
	if (setting->epsbg != tag) {
		setting->epsbg = tag;
		changeFlag = YES;
		[revertButton setEnabled:changeFlag];
		[panel setDocumentEdited:YES];
	}
}

- (void)setFontname:(id)sender
{
	NSFontManager *fontManager = [NSFontManager sharedFontManager];
	[panel makeFirstResponder:panel];
	[fontManager setSelectedFont:setting->txfont isMultiple:NO];
	[fontManager orderFrontFontPanel:self];
}

/* Action method from Font Panel */
- (void)changeFont:(id)fontManager
{
	NSFont *w;

	w = [fontManager convertFont:setting->txfont];
	[[fontManager fontPanel:NO] performClose:self];
	if (w == setting->txfont)
		return;
	setting->txfont = w;
	[fontnameTX setStringValue:[setting->txfont fontName]];
	changeFlag = YES;
	[revertButton setEnabled:changeFlag];
	[panel setDocumentEdited:YES];
}


- (void)setAllValue:(id)sender
{
	if (!changeFlag)
		return;
	[setting->bgcolor release];
	[(setting->bgcolor = [backgColorWell color]) retain];
	[setting->txcolor release];
	[(setting->txcolor = [textColorWell color]) retain];
	[setting->txbgcolor release];
	[(setting->txbgcolor = [textBgColorWell color]) retain];
	setting->transparent = [textTransp floatValue];
	setting->usetxbg = ([textBgSW state] != 0);
	setting->baseline = [baselineSL intValue];
	setting->fontsize = [fontsizeSL intValue];
	[scenarioCtrl settingChanged:self];
	changeFlag = NO;
	[revertButton setEnabled:changeFlag];
	[panel setDocumentEdited:NO];
}

- (void)revert:(id)sender
{
	[self activateAndOrderFront:self];
}

- (void)saveAsDefault:(id)sender
{
	[self setAllValue:sender];
	[[PrefCtrl defaultPref] setAsDefaultValue: setting];
}

- (void)useDefault:(id)sender
{
	struct setinfo *info, sav;

	info = [[PrefCtrl defaultPref] defaultSetting];
	sav = *setting;		/* save current value */
	*setting = *info;
	[self activateAndOrderFront:self];
	*setting = sav;
	changeFlag = YES;
	[revertButton setEnabled:changeFlag];
	[panel setDocumentEdited:YES];
}

/* NSWinodow's Delegate */
- (BOOL)windowShouldClose:(id)sender
{
	if (changeFlag) {
		NSBeep();
		return NO;
	}
	return YES;
}

@end
