#import "SettingDictionary.h"
#import <Foundation/NSArray.h>
#import <Foundation/NSString.h>
#import <Foundation/NSBundle.h>
#import "NSColorAdditional.h"
#import <stdlib.h>


struct setinfo *copySetinfo(struct setinfo *info)
{
	struct setinfo *p;
	p = (struct setinfo *)malloc(sizeof(struct setinfo));
	*p = *info;
	[p->bgcolor retain];
	[p->txcolor retain];
	[p->txbgcolor retain];
	return p;
}

void freeSetinfo(struct setinfo *info)
{
	[info->bgcolor release];
	[info->txcolor release];
	[info->txbgcolor release];
	free((void *)info);
}

int realFontSize(int fsize)
{
	static unsigned char sz[] = { 10, 11, 12, 14, 16, 18, 20, 24, 28 };
	if (fsize < 1) fsize = 1;
	else if (fsize > 9) fsize = 9;
	return (int)sz[fsize - 1];
}

@implementation SettingDictionary

- (void)dealloc
{
	[dictionary release];
	[super dealloc];
}

- (void)forwardInvocation:(NSInvocation *)invocation
{
	[invocation invokeWithTarget:dictionary];
	/* Forward all message to dictionary */
	/* This method does not work... Bug of ObjC Runtime Library ?? */
}

- (id)initWithDictionary:(NSDictionary *)otherDictionary
{
	NSMutableArray *marr;
	BOOL	uflag;
	id	tr;
	static unsigned char bdefs[3] = { 150, 150, 232 };
	static unsigned char tdefs[3] = { 0, 0, 0 };
	static unsigned char tbdefs[3] = { 255, 255, 255 };

	dictionary = [[NSMutableDictionary alloc] initWithDictionary:otherDictionary];
	if ([dictionary objectForKey: SBackgColor] == nil) {
		marr = [NSColor arrayWithRGBColors:bdefs];
		[dictionary setObject:marr forKey:SBackgColor];
	}
	if ([dictionary objectForKey: STextColor] == nil) {
		marr = [NSColor arrayWithRGBColors:tdefs];
		[dictionary setObject:marr forKey:STextColor];
	}
	uflag = YES;
	if ([dictionary objectForKey: STextBgColor] == nil) {
		marr = [NSColor arrayWithRGBColors:tbdefs];
		[dictionary setObject:marr forKey:STextBgColor];
		uflag = NO;
	}
	if ([dictionary objectForKey: SUseTextBg] == nil)
		[dictionary setObject:(uflag ? @"YES" : @"NO")
			forKey:SUseTextBg];
	if ([dictionary objectForKey: SEPSBackg] == nil)
		[dictionary setObject:EPS_K_White forKey:SEPSBackg];
	if ((tr = [dictionary objectForKey: STransparent]) == nil)
		[dictionary setObject:@"0.6" forKey:STransparent];
	else if ([tr floatValue] < TranspUnderLimit)
		[dictionary setObject:@"NO" forKey:SUseTextBg];
	if ([dictionary objectForKey: SBaseline] == nil)
		[dictionary setObject:@"0" forKey:SBaseline];
	if ([dictionary objectForKey: SFontSize] == nil)
		[dictionary setObject:@"3" forKey:SFontSize];
	if ([dictionary objectForKey: STextFont] == nil) {
		NSString *deffont = NSLocalizedString(@"Helvetica", Font);
		[dictionary setObject:deffont forKey:STextFont];
	}
	return self;
}

- (id)dictionary
{
	return dictionary;
}

- (id)initWithSettingInfo:(struct setinfo *)info
{
	id	arr, str;

	dictionary = [[NSMutableDictionary alloc] initWithCapacity:5];
	arr = [info->bgcolor RGBArrayForColor];
	[dictionary setObject:arr forKey:SBackgColor];
	arr = [info->txcolor RGBArrayForColor];
	[dictionary setObject:arr forKey:STextColor];
	arr = [info->txbgcolor RGBArrayForColor];
	[dictionary setObject:arr forKey:STextBgColor];
	str = [NSString stringWithFormat:@"%4.2f", info->transparent];
	[dictionary setObject:str forKey:STransparent];
	str = info->usetxbg ? @"YES" : @"NO";
	[dictionary setObject:str forKey:SUseTextBg];
	if (info->epsbg == 0) str = @"NO";
	else str = (info->epsbg == EPS_BG_White) ? EPS_K_White: EPS_K_Text;
	[dictionary setObject:str forKey:SEPSBackg];
	str = [NSString stringWithFormat:@"%d", info->baseline];
	[dictionary setObject:str forKey:SBaseline];
	str = [NSString stringWithFormat:@"%d", info->fontsize];
	[dictionary setObject:str forKey:SFontSize];
	str = (info->txfont) ? [info->txfont fontName]
		: NSLocalizedString(@"Helvetica", Font);
	[dictionary setObject:str forKey:STextFont];
	return self;
}

- (struct setinfo *)info
	/* Note: Objects of NSColor in setinfo IS NOT retain'ed. */
	/* See copySetinfo() also. */
{
	id	arr, str;

	arr = [dictionary objectForKey:SBackgColor];
	tmpinfo.bgcolor = [NSColor colorWithRGBArray:arr];
	arr = [dictionary objectForKey:STextColor];
	tmpinfo.txcolor = [NSColor colorWithRGBArray:arr];
	arr = [dictionary objectForKey:STextBgColor];
	tmpinfo.txbgcolor = [NSColor colorWithRGBArray:arr];
	tmpinfo.transparent
		= [[dictionary objectForKey:STransparent] floatValue];
	tmpinfo.usetxbg = (tmpinfo.transparent < TranspUnderLimit) ? NO
	: [[dictionary objectForKey:SUseTextBg] isEqualToString:@"YES"];
	str = [dictionary objectForKey:SEPSBackg];
	if ([str isEqualToString:@"NO"])
		tmpinfo.epsbg = EPS_BG_none;
	else
		tmpinfo.epsbg = ([str isEqualToString:EPS_K_White])
				? EPS_BG_White : EPS_BG_Text;
	tmpinfo.baseline = [[dictionary objectForKey:SBaseline] intValue];
	tmpinfo.fontsize = [[dictionary objectForKey:SFontSize] intValue];
	str = [dictionary objectForKey:STextFont];
	tmpinfo.txfont
	    = [NSFont fontWithName:str size:realFontSize(tmpinfo.fontsize)];
	return &tmpinfo;
}

- (id)objectForKey:(id)aKey
{
	return [dictionary objectForKey:aKey];
}

@end
