#import "ToyProjCtrl.h"
#import <AppKit/NSApplication.h>
#import <AppKit/NSOpenPanel.h>
#import <AppKit/NSSavePanel.h>
#import <AppKit/NSText.h>
#import <AppKit/NSImage.h>
#import <AppKit/NSImageView.h>
#import <AppKit/NSScrollView.h>
#import <AppKit/NSFileWrapper.h>
#import <AppKit/NSMenu.h>
#import <Foundation/NSData.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSBundle.h>	/* LocalizedString */
#import <stdio.h>
#import "ScenarioCtrl.h"
#import "PrefCtrl.h"
#import "InspectorCtrl.h"
#import "CoverSlide.h"
#import "commtypes.h"


@implementation ToyProjCtrl

static NSSavePanel *svp = nil;
static NSString *sdir = nil;
static NSOpenPanel *opp = nil;
static NSString *odir = nil;
static NSArray *fileTypes = nil;
static id winList = nil;

/* Local Method */
- (void)addToList:(id)scenario
{
	if (winList == nil)
		winList = [[NSMutableArray alloc] initWithCapacity:1];
	[winList addObject: scenario];
}

/* Local Method */
- (id)isOpenedNow:(NSString *)fname
{
	int	i, n;
	id	obj;

	if (winList == nil)
		return nil;
	n = [winList count];
	for (i = 0; i < n; i++) {
		obj = [winList objectAtIndex:i];
		if ([[obj filename] isEqualToString:fname])
			return obj;
	}
	return nil;
}

/* Local Method */
- (BOOL)openScenario:(NSString *)fn newly:(BOOL)flag
{
	ScenarioCtrl *newscn;
	BOOL	result;

	if ((newscn = [[ScenarioCtrl alloc] init]) == nil)
		return NO;		/* Error */
	result = flag ?
		[newscn setAndCreateDir: fn] : [newscn setAndOpenDoc: fn];
	if (!result) {
		[newscn release];
		return NO;
	}
	[newscn setController:self];
	[self addToList: newscn];
	return YES;
}

- (void)openFile:(id)sender
{
	NSString *fn, *dr;
	NSString *msg;
	NSArray *farr;
	int	n, i;
	id	scen;

	if (opp == nil) {
		fileTypes = [[NSArray arrayWithObject:EXTENSION] retain];
		opp = [NSOpenPanel openPanel];
		[opp setTitle:NSLocalizedString(@"Open Doc", Open_Doc)];
		[opp setTreatsFilePackagesAsDirectories:NO];
		[opp setAllowsMultipleSelection:YES];
		odir = [NSHomeDirectory() copyWithZone:[self zone]];
	}
	if ([opp runModalForDirectory:odir file:nil types:fileTypes] == NSCancelButton)
		return;

	farr = [opp filenames];
	dr = [opp directory];
	if (![odir isEqualToString: dr]) {
		[odir release];
		odir = [dr retain];
	}

	n = [farr count];
	for (i = 0; i < n; i++) {
		fn = [farr objectAtIndex: i];
		if (![[fn pathExtension] isEqualToString:EXTENSION]) {
			msg = NSLocalizedString(@"Illegal file type", Err_Type);
			NSRunAlertPanel(NSLocalizedString(@"Error", Error),
				@"%@ :\n%@", @"", nil, nil, msg, fn);
			continue;
		}
		if ((scen = [self isOpenedNow: fn]) != nil) {
			[scen makeKeyAndOrderFront:self];
			continue;
		}
		if (![self openScenario:fn newly:NO]) {
			msg = NSLocalizedString(@"Can not read", Err_Read);
			NSRunAlertPanel(NSLocalizedString(@"Error", Error),
				@"%@ :\n%@", @"", nil, nil, msg, fn);
		}
	}
}

- (void)newDocument:(id)sender
{
	NSString *fn, *dr;

	if (svp == nil) {
		svp = [NSSavePanel savePanel];
		[svp setTreatsFilePackagesAsDirectories:NO];
		[svp setRequiredFileType:EXTENSION];
		sdir = [NSHomeDirectory() copyWithZone:[self zone]];
	}
	[svp setTitle:NSLocalizedString(@"Create New Doc", Create_Doc)];
	if ([svp runModalForDirectory:sdir file:@""] == NSCancelButton)
		return;

	fn = [svp filename];
	dr = [svp directory];
	if (![sdir isEqualToString: dr]) {
		[sdir release];
		sdir = [dr retain];
	}
	(void)[self openScenario:fn newly:YES];
}

/* Local Method */
- (id)getMainDocument
{
	int	n, i;
	id	obj;

	if (winList == nil || (n = [winList count]) <= 0)
		return nil;
	for (i = 0; i < n; i++) {
		obj = [winList objectAtIndex:i];
		if ([[obj window] isMainWindow])
			return obj;
	}
	return nil;
}

/* Local Method */
- (BOOL)isString:(NSString *)dr ancestorOf:(NSString *)fname
{
	int	n, i;
	id darr = [[dr stringByStandardizingPath] pathComponents];
	id farr = [[fname stringByStandardizingPath] pathComponents];
	n = [darr count];
	if (n > [farr count])
		return NO;
	for (i = 0; i < n; i++)
		if (![[darr objectAtIndex:i]
			isEqualToString:[farr objectAtIndex:i]])
			return NO;
	return YES;
}

- (void)saveFile:(id)sender
{
	id	obj;

	if ((obj = [self getMainDocument]) == nil) {
		NSBeep();
		return;
	}
	[obj save];
}

- (void)saveFileAs:(id)sender
{
	id	obj;
	NSString *fname;
	NSString *fn, *dr;

	if ((obj = [self getMainDocument]) == nil) {
		NSBeep();
		return;
	}
	fname = [obj filename];
	if (svp == nil) {
		svp = [NSSavePanel savePanel];
		[svp setTreatsFilePackagesAsDirectories:NO];
		[svp setRequiredFileType:EXTENSION];
		sdir = [fname stringByDeletingLastPathComponent];
		[sdir retain];
	}
	[svp setTitle:NSLocalizedString(@"Save As...", Save_Doc_As)];
	fn = [fname lastPathComponent];
	if ([svp runModalForDirectory:sdir file:fn] == NSCancelButton)
		return;

	fn = [svp filename];
	dr = [svp directory];
	if ([fn isEqualToString: fname]) {
		[obj save];
		return;
	}
	if (![self isString:dr ancestorOf:fname]) {
		NSRunAlertPanel(NSLocalizedString(@"Error", Error),
			NSLocalizedString(@"Be an ancestor", Err_Ancestor),
			@"", nil, nil);
		return;
	}
	if (![sdir isEqualToString: dr]) {
		[sdir release];
		sdir = [dr retain];
	}
	[obj saveAs:fn];
}

- (void)saveAll:(id)sender
{
	int	n, i;

	if ((n = [winList count]) <= 0) {
		NSBeep();
		return;
	}
	for (i = 0; i < n; i++)
		[[winList objectAtIndex:i] saveIfNeeded];
}

- (void)reloadFile:(id)sender
{
	id	obj;

	if ((obj = [self getMainDocument]) != nil)
		[obj reload];
	else
		NSBeep();
}

- (void)activateInspector:(id)sender
{
	id	obj;

	if ((obj = [self getMainDocument]) != nil)
		[obj activateInspector:self];
	else
		NSBeep();
}

- (void)activatePreferences:(id)sender
{
	[[PrefCtrl defaultPref] makeKeyAndOrderFront:sender];
}

- (void)activateCoverSheet:(id)sender
{
	static Class coverslide = Nil;
	id	obj;

	if (!coverslide) {	/* Load Class CoverSlide */
		NSBundle *bundle = [NSBundle mainBundle];
		NSString *path = [bundle pathForResource:@"CoverSheet"
				ofType:@"bundle"];
		bundle = [NSBundle bundleWithPath: path];
		if (bundle == nil) /* ERROR */
			return;
		coverslide = [bundle classNamed:@"CoverSlide"];
	}
	obj = [[coverslide alloc] initForCoverSheet];
	[obj coverSheet:self];
	/* obj is release'd when it is closed */
}

- (void)scenarioShouldClose:(id)scenario
{
	[winList removeObject: scenario];
	if ([winList count] == 0)
		[[InspectorCtrl defaultController] setCurrentTable:nil];
}

/* Based on NSMenuActionResponder's Protocol */
- (BOOL)validateMenuItem:(NSMenuItem *)anItem
{
	if ([anItem tag] == 0)
		return YES;
	return (winList != nil && [winList count] > 0);
}

/* NSApplication's Delegate */
- (void)applicationDidFinishLaunching:(NSNotification *)notification
{
	[version setStringValue:
		[NSString stringWithCString:CurrentVERSION]];
}

/* NSApplication's Delegate */
- (BOOL)application:(NSApplication *)theApp openFile:(NSString *)filename
{
	id	scen;

	if (![[filename pathExtension] isEqualToString:EXTENSION])
		return NO;
	if ((scen = [self isOpenedNow:filename]) != nil) {
		[scen makeKeyAndOrderFront:self];
		return YES;
	}
	return [self openScenario:filename newly:NO];
}

/* Local Method */
- (BOOL)askAppCanTerminate:(NSApplication *)sender
{
	int	n, i, r;
	id	obj;
	BOOL	isEdited = NO;

	if (winList == nil || (n = [winList count]) <= 0)
		return YES;
	for (i = 0; i < n; i++) {
		obj = [winList objectAtIndex:i];
		[obj quitShow];
		if ([[obj window] isDocumentEdited])
			isEdited = YES;
	}
	if (isEdited == NO)
		return YES;

	r = NSRunAlertPanel(
		NSLocalizedString(@"Quit", Quit),
		NSLocalizedString(@"There are edited windows", EditedWindows),
		NSLocalizedString(@"Review unsaved", Review),
		NSLocalizedString(@"Quit anyway", Anyway),
		NSLocalizedString(@"Cancel", Cancel)
	);
	switch (r) {
	case NSAlertDefaultReturn:	/* Review */
		break;
	case NSAlertAlternateReturn:	/* Quit anyway */
		return YES;
	case NSAlertOtherReturn:	/* Cancel */
	default:
		return NO;
	}
	for (i = 0; i < n; i++)
		[[winList objectAtIndex:i] appShouldQuit];
	return YES;
}

/* NSApplication's Delegate */
- (BOOL)applicationShouldTerminate:(NSApplication *)sender
{
	if ([self askAppCanTerminate:sender]) {
		[InspectorCtrl appShouldTerminate];
		[PrefCtrl appShouldTerminate];
		return YES;
	}
	return NO;
}

@end
