#import "sheetInfo.h"
#import <Foundation/NSString.h>

void clearOption(opt_t *valp)
{
	valp->overlap = valp->imageSz = valp->title
	 = valp->backgflag = valp->background = valp->notextclr
	 = valp->texttransp = valp->baseline = valp->fontsize
	 = valp->mark = 0;
}

void convStringToOption(NSString *str, opt_t *valp)
{
	int	i, cc;
	const char *p;

	clearOption(valp);
	for (p = [str cString]; *p; p++) {
		switch (*p) {
		case 'A':  case 'a':	/* alpha */
			valp->texttransp = 1;
			break;
		case 'B':  case 'b':	/* background */
			cc = *(p+1);
			for (i = 0; i < 5; i++) {
				if (cc == "ctbfv"[i])
					break;
			}
			if (i <= 4) p++;
			else i = 4;
			valp->backgflag = 1;
			valp->background = i;
			return;	/* not break */
		case 'M':  case 'm':	/* mark */
			cc = *(p+1);
			for (i = 0; i < 10; i++) {
				if (cc == "kbcgmopryw"[i])
					break;
			}
			if (i <= 9) p++;
			else i = 5;	/* orange */
			valp->mark = 0x10 | i;
			break;
		case 'O':  case 'o':	/* overlap */
			valp->overlap = 1;
			break;
		case 'X':  case 'x':	/* Image Size */
			cc = *(p+1);
			if (cc >= '1' && cc <= '9') {
				i = cc - '0';
				valp->imageSz = (i > 4) ? im_Integer : i;
				p++;
			}else if (cc == 'i') {
				valp->imageSz = im_Integer;
				p++;
			}else if (cc == 'f') {
				valp->imageSz = im_Screen;
				p++;
			}else if (cc == 'v') {
				valp->imageSz = im_Screen + 1;
				p++;
			}
			break;
		case 'R':  case 'r':	/* Raise Baseline */
			cc = *(p+1);
			if (cc >= '1' && cc <= '9') {
				valp->baseline = cc - '0';
				p++;
			}else
				valp->baseline = 2;
			break;
		case 'S':  case 's':	/* Font Size */
			cc = *(p+1);
			if (cc >= '1' && cc <= '9') {
				valp->fontsize = cc - '0';
				p++;
			}else
				valp->fontsize = 3;
			break;
		case 'T':  case 't':	/* Title */
			cc = *(p+1);
			if (cc == 'u' || cc == 'U') {
				valp->title = 1; /* Upper */
				p++;
			}else {
				valp->title = 2; /* Lower */
				if (cc == 'l' || cc == 'L')
					p++;
			}
			break;
		case 'C':  case 'c':	/* Color */
			valp->notextclr = 1;
			break;
		/* Other chars are ignored */
		} /* End of switch */
	}
}

NSString *convToString(const opt_t *valp)
{
	int	k, idx, cc;
	char	buf[16];

	if (valp->backgflag) {
		k = (valp->background) % 5;
		buf[0] = 'B';
		buf[1] = "ctbfv"[k];
		buf[2] = 0;
		return [NSString stringWithCString:buf];
	}

	idx = 0;
	if (valp->overlap)
		buf[idx++] = 'O';
	if (valp->mark) {
		buf[idx++] = 'M';
		buf[idx++] = "kbcgmopryw"[ valp->mark & 0x0f ];
	}
	if (valp->imageSz != im_Auto) {
		buf[idx++] = 'X';
		if (valp->imageSz < im_Integer)
			cc = '0' + valp->imageSz;
		else if (valp->imageSz == im_Integer)
			cc = 'i';
		else
			cc = (valp->imageSz == im_Screen) ? 'f' : 'v';
		buf[idx++] = cc;
	}
	if (valp->baseline != 0) {
		buf[idx++] = 'R';
		buf[idx++] = '0' + valp->baseline;
	}
	if (valp->fontsize != 0) {
		buf[idx++] = 'S';
		buf[idx++] = '0' + valp->fontsize;
	}
	if (valp->title != 0) {
		buf[idx++] = 'T';
		if (valp->title == 1)
			buf[idx++] = 'u';
	}
	if (valp->notextclr != 0)
		buf[idx++] = 'C';
	if (valp->texttransp != 0)
		buf[idx++] = 'A';
	buf[idx] = 0;
	return [NSString stringWithCString:buf];
}
